package cn.com.qmth.examcloud.core.oe.student.service.impl;

import java.util.Date;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.core.oe.student.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.student.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordDataService;
import cn.com.qmth.examcloud.support.cache.bean.CourseCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.ExamSettingsCacheBean;
import cn.com.qmth.examcloud.support.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;
import cn.com.qmth.examcloud.support.examing.ExamingSession;
import cn.com.qmth.examcloud.support.redis.RedisKeyHelper;
import cn.com.qmth.examcloud.web.redis.RedisClient;

/**
 * @author chenken
 * @date 2018/8/15 11:16
 * @company QMTH
 * @description 考试记录数据服务实现
 */
@Service("examRecordDataService")
public class ExamRecordDataServiceImpl implements ExamRecordDataService {

    @Autowired
    private RedisClient redisClient;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Transactional
    @Override
    public ExamRecordData createExamRecordData(ExamingSession examingSession, ExamSettingsCacheBean examBean,
                                               CourseCacheBean courseBean, String basePaperId) {
        ExamRecordDataEntity examRecordData = new ExamRecordDataEntity();
        examRecordData.setExamId(examBean.getId());
        examRecordData.setExamType(ExamType.valueOf(examBean.getExamType()));

        examRecordData.setExamStudentId(examingSession.getExamStudentId());
        examRecordData.setStudentId(examingSession.getStudentId());
        examRecordData.setOrgId(examingSession.getOrgId());
        examRecordData.setRootOrgId(examingSession.getRootOrgId());

        examRecordData.setCourseId(courseBean.getId());
        examRecordData.setBasePaperId(basePaperId);

        examRecordData.setPaperType(examingSession.getPaperType());

        examRecordData.setStartTime(new Date());
        examRecordData.setIsContinued(false);
        examRecordData.setContinuedCount(0);
        examRecordData.setFaceSuccessCount(0);
        examRecordData.setFaceTotalCount(0);
        examRecordData.setFaceFailedCount(0);
        examRecordData.setFaceStrangerCount(0);
        examRecordData.setExamRecordStatus(ExamRecordStatus.EXAM_ING);
        examRecordDataRepo.save(examRecordData);
        ExamRecordData bean = of(examRecordData);
        //存入redis
        saveExamRecordDataCache(examRecordData.getId(), bean);
        return bean;
    }

    @Override
    public void saveExamRecordDataCache(Long examRecordDataId, ExamRecordData data) {
        String key = RedisKeyHelper.getBuilder().examRecordDataKey(examRecordDataId);
        redisClient.set(key + examRecordDataId, data, -1);
    }

    @Override
    public ExamRecordData getExamRecordDataCache(Long examRecordDataId) {
        String key = RedisKeyHelper.getBuilder().examRecordDataKey(examRecordDataId);
        return redisClient.get(key + examRecordDataId, ExamRecordData.class);
    }

    @Override
    public void deleteExamRecordDataCache(Long examRecordDataId) {
        String key = RedisKeyHelper.getBuilder().examRecordDataKey(examRecordDataId);
        redisClient.delete(key + examRecordDataId);
    }

    private ExamRecordData of(ExamRecordDataEntity et) {
        ExamRecordData bean = new ExamRecordData();
        bean.setId(et.getId());
        bean.setExamId(et.getId());
        bean.setExamType(et.getExamType());

        bean.setExamStudentId(et.getExamStudentId());
        bean.setStudentId(et.getStudentId());
        bean.setCourseId(et.getCourseId());
        bean.setOrgId(et.getOrgId());
        bean.setRootOrgId(et.getRootOrgId());

        bean.setBasePaperId(et.getBasePaperId());

        bean.setPaperType(et.getPaperType());

        bean.setStartTime(et.getStartTime());
        bean.setEndTime(et.getEndTime());
        bean.setUsedExamTime(et.getUsedExamTime());
        bean.setIsContinued(et.getIsContinued());
        bean.setContinuedCount(et.getContinuedCount());
        bean.setIsExceed(et.getIsExceed());
        bean.setFaceSuccessCount(et.getFaceSuccessCount());
        bean.setFaceFailedCount(et.getFaceFailedCount());
        bean.setFaceStrangerCount(et.getFaceStrangerCount());
        bean.setFaceTotalCount(et.getFaceTotalCount());
        bean.setExamRecordStatus(et.getExamRecordStatus());
        bean.setHandInExamType(et.getHandInExamType());
        bean.setFaceVerifyResult(et.getFaceVerifyResult());
        bean.setCleanTime(et.getCleanTime());
        bean.setIsWarn(et.getIsWarn());
        bean.setIsAudit(et.getIsAudit());
        bean.setIsIllegality(et.getIsIllegality());
        bean.setTotalScore(et.getTotalScore());
        bean.setObjectiveScore(et.getObjectiveScore());
        bean.setObjectiveAccuracy(et.getObjectiveAccuracy());
        bean.setSubjectiveScore(et.getSubjectiveScore());
        bean.setSuccPercent(et.getSuccPercent());
        return bean;
    }
}
